@@ -13,6 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import json
 import sys
 import textwrap
 import traceback
@@ -24,6 +25,8 @@ import six
 
 from oslo_upgradecheck._i18n import _
 
+CONF = None
+
 
 class Code(enum.IntEnum):
     """Status codes for the upgrade check command"""
@@ -75,6 +78,7 @@ class UpgradeCommands(object):
     done through the sqlalchemy query language directly like the database
     schema migrations.
     """
+    display_title = _('Upgrade Check Results')
     _upgrade_checks = ()
 
     def _get_details(self, upgrade_check_result):
@@ -91,6 +95,7 @@ class UpgradeCommands(object):
 
         :returns: Code
         """
+        global CONF
         return_code = Code.SUCCESS
         # This is a list if 2-item tuples for the check name and it's results.
         check_results = []
@@ -120,42 +125,77 @@ class UpgradeCommands(object):
         # NOTE(bnemec): We use six.text_type on the translated string to
         # force immediate translation if lazy translation is in use.
         # See lp1801761 for details.
-        t = prettytable.PrettyTable([six.text_type(_('Upgrade Check Results'))
-                                     ],
-                                    hrules=prettytable.ALL)
-        t.align = 'l'
-        for name, result in check_results:
-            cell = (
-                _('Check: %(name)s\n'
-                  'Result: %(result)s\n'
-                  'Details: %(details)s') %
-                {
-                    'name': name,
-                    'result': UPGRADE_CHECK_MSG_MAP[result.code],
-                    'details': self._get_details(result),
-                }
-            )
-            t.add_row([cell])
-        print(t)
+
+        # Since registering opts can be overridden by consuming code, we can't
+        # assume that our locally defined option exists.
+        if (hasattr(CONF, 'command') and hasattr(CONF.command, 'json') and
+                CONF.command.json):
+            output = {'name': six.text_type(self.display_title), 'checks': []}
+            for name, result in check_results:
+                output['checks'].append(
+                    {'check': name,
+                     'result': result.code,
+                     'details': result.details}
+                )
+            print(json.dumps(output))
+        else:
+            t = prettytable.PrettyTable([six.text_type(self.display_title)],
+                                        hrules=prettytable.ALL)
+            t.align = 'l'
+            for name, result in check_results:
+                cell = (
+                    _('Check: %(name)s\n'
+                      'Result: %(result)s\n'
+                      'Details: %(details)s') %
+                    {
+                        'name': name,
+                        'result': UPGRADE_CHECK_MSG_MAP[result.code],
+                        'details': self._get_details(result),
+                    }
+                )
+                t.add_row([cell])
+            print(t)
 
         return return_code
 
 
-def _register_cli_options(conf, upgrade_command):
+def register_cli_options(conf, upgrade_command):
     """Set up the command line options.
 
     Adds a subcommand to support 'upgrade check' on the command line.
 
+    :param conf: An oslo.confg ConfigOpts instance on which to register the
+                 upgrade check arguments.
+    :param upgrade_command: The UpgradeCommands instance.
     """
     def add_parsers(subparsers):
         upgrade_action = subparsers.add_parser('upgrade')
         upgrade_action.add_argument('check')
         upgrade_action.set_defaults(action_fn=upgrade_command.check)
+        upgrade_action.add_argument(
+            '--json',
+            action='store_true',
+            help='Output the results in JSON format. Default is to print '
+                 'results in human readable table format.')
 
     opt = cfg.SubCommandOpt('command', handler=add_parsers)
     conf.register_cli_opt(opt)
 
 
+def run(conf):
+    """Run the requested command.
+
+    :param conf: An oslo.confg ConfigOpts instance on which the upgrade
+                 commands have been previously registered.
+    """
+    try:
+        return conf.command.action_fn()
+    except Exception:
+        print(_('Error:\n%s') % traceback.format_exc())
+        # This is 255 so it's not confused with the upgrade check exit codes.
+        return 255
+
+
 def main(conf, project, upgrade_command,
          argv=sys.argv[1:],
          default_config_files=None):
@@ -177,17 +217,13 @@ def main(conf, project, upgrade_command,
                                  the search behavior in oslo.config.
 
     """
-    _register_cli_options(conf, upgrade_command)
+    global CONF
+    register_cli_options(conf, upgrade_command)
 
     conf(
         args=argv,
         project=project,
         default_config_files=default_config_files,
     )
-
-    try:
-        return conf.command.action_fn()
-    except Exception:
-        print(_('Error:\n%s') % traceback.format_exc())
-        # This is 255 so it's not confused with the upgrade check exit codes.
-        return 255
+    CONF = conf
+    return run(conf)
